﻿using System;
using System.IO.Ports;
using System.Threading;
using System.Windows.Forms;

// Это тест работает для TRexJr, скорость 19200

namespace TRexComConsole {
	class Program {
		#region Constants
		private const String PORT_NAME_WINDOWS = "COM9";
		private const String PORT_NAME_LINUX = "/dev/ttyUSB0";
		#endregion

		#region Variables
		private SerialPort serialPort1;
		/// <summary>
		/// Default device number for the TReX and TReX Jr
		/// </summary>
		private Int32 _deviceNum = 7;

		/// <summary>
		/// Does the TReX echo everything you transmit (will be true if you use RS-232)
		/// </summary>
		private Boolean _isSerialEcho;
		#endregion

		#region Main
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			new Program()._run();
		}
		#endregion

		#region Utils
		private void _run() {
			this.serialPort1 = new SerialPort();
			this.serialPort1.PortName = (this._isLinux() ? PORT_NAME_LINUX : PORT_NAME_WINDOWS);
			this.serialPort1.BaudRate = 19200;
			this.serialPort1.ReadTimeout = 150;
			this._getSignature();

			this._motor1_F();
			Thread.Sleep(3000);
			this._motor1_0();
			Thread.Sleep(1000);

			this._motor1_B();
			Thread.Sleep(3000);
			this._motor1_0();

			if (!this._isLinux()) { Console.ReadLine(); }
		}

		private Byte[] _sendCommandPacket(Byte[] buffer, int readBytes, ref string str) {
			/* This function serially transmits the specified command packet and then waits to receive
			 * the specified number of bytes in response.  Bytes to transmit are provided via the
			 * 'packet' array, and received bytes are stored in the 'packet' array.
			 * It blocks execution until the desired number of bytes have been received from the TReX
			 * or until the serial read times out (150 ms).
			 *   packet - an array of bytes to transmit to the TReX; when the function is through,
			 *            this array will contain any bytes received from the TReX, so the size of
			 *            this array must be greater than or equal to Max(sendBytes, readBytes)
			 *   sendBytes - the number of bytes in the command packet
			 *   readBytes - the number of bytes to try to receive from the TReX in response
			 *   str - a string that contains information about what was sent and received;
			 *         used for debugging/feedback purposes
			 */
			Byte[] packet = new Byte[32];
			int i;
			str = "";

			try {
				// if there are any unread bytes in the read buffer, they are junk
				//  read them now so the buffer is clear to receive anything the TReX
				//  might send back in response to the command
				if (!serialPort1.IsOpen) { serialPort1.Open(); }
				while (serialPort1.BytesToRead > 0)
					serialPort1.ReadByte();

				str += " TX={ ";
				Int32 sendBytes = buffer.Length;

				for (i = 0; i < buffer.Length; i++) { packet[i + 2] = buffer[i]; }
				packet[0] = 0x80;
				packet[1] = (Byte) _deviceNum;
				packet[2] -= 0x80;     // clear MSB of command byte
				sendBytes += 2;

				for (i = 0; i < sendBytes; i++)
					str += packet[i].ToString("X2") + " ";
				serialPort1.Write(packet, 0, sendBytes);
				if (_isSerialEcho) {
					str += "};  Echo={ ";
					for (i = 0; i < sendBytes; i++)
						str += serialPort1.ReadByte().ToString("X2") + " ";
				}
				str += "};  RX={ ";
				Byte[] result = new Byte[readBytes];
				for (i = 0; i < readBytes; i++) {
					result[i] = (Byte) serialPort1.ReadByte();
					str += packet[i].ToString("X2") + " ";
				}
				str += "}";
				if (serialPort1.IsOpen) { serialPort1.Close(); }
				return result;
			}
			catch (Exception ex) {
				str += " " + ex.Message;
				return null;
			}
		}

		private Boolean _isLinux() {
			return (Application.ExecutablePath.IndexOf("/") >= 0);
		}

		private void _getSignature() {
			try {
				Console.Clear();
				
				this.serialPort1.Open();

				Byte[] buffer = new Byte[] { 0xFF };
				// Send the null command
				this.serialPort1.Write(buffer, 0, buffer.Length);
				try {
					// Look for an echo
					Int32 data = this.serialPort1.ReadByte();
					if (data == 255) {
						this._isSerialEcho = true;
					}
				}
				catch (TimeoutException) {
					this._isSerialEcho = false;     // this line will be executed if we are using TTL serial
				}

				// Read the device signature
				String str = "";
				Byte[] result = this._sendCommandPacket(new Byte[] { 0x81 }, 7, ref str);
				if (result == null) {
					Console.Write("failure\r\n" + str);
				}
				else {
					Console.WriteLine("" + str);
					for (int i = 0; i < 7; i++)
						Console.Write((char) result[i]);
					Console.WriteLine("");
				}
			}
			catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}
			if (serialPort1.IsOpen) { this.serialPort1.Close(); }
		}

		private void _motor1_0() {
			String str = "";
			this._sendCommandPacket(new Byte[] { 0xD0, 0x00, 0x00 }, 0, ref str);
			Console.WriteLine(str);
		}

		private void _motor2_0() {
			String str = "";
			this._sendCommandPacket(new Byte[] { 0xD0, 0x00, 0x00 }, 0, ref str);
			Console.WriteLine(str);
		}

		private void _motor1_F() {
			String str = "";
			// 1=>2
			this._sendCommandPacket(new Byte[] { 0xD0 | 0x08, 0, 30 }, 0, ref str);
			Console.WriteLine(str);
		}

		private void _motor2_F() {
			String str = "";
			// 2=>1
			this._sendCommandPacket(new Byte[] { 0xD0 | 0x02, 30, 0 }, 0, ref str);
			Console.WriteLine(str);
		}

		private void _motor1_B() {
			String str = "";
			// 1=>2
			this._sendCommandPacket(new Byte[] { 0xD0 | 0x04, 0, 30 }, 0, ref str);
			Console.WriteLine(str);
		}

		private void _motor2_B() {
			String str = "";
			// 2=>1
			this._sendCommandPacket(new Byte[] { 0xD0 | 0x01, 30, 0 }, 0, ref str);

			// FB => 
			//this._sendCommandPacket(new Byte[] { 0xD0 | 0x08 | 0x01, 50, 50 }, 0, ref str);

			Console.WriteLine(str);
		}
		#endregion
	}
}